import { IRealDrawBoardEvents } from '../../constants/events/RealDrawBoardEvents';
import { RealDrawBoardOptions, RealDrawBoardSettings } from '../../types/RealDrawBoardTypes';
import { Coordinate, Stroke } from '../../types/RealRendererTypes';
import { RealRenderer } from '../RealRenderer/RealRenderer';
import { changeTool, changeToolSetting, clear, clearPreview, _resetBoard } from './boardManip';
import { Tool, Tools, ToolSettings } from './tools/tools';
import { _getMouseCoords, _getTouchCoords } from './_coords';
import { _addDOMEvents, _removeDOMEvents } from './_DOMEvents';
export * from '../../constants/defaults/RealDrawBoardDefaults';
export * as RealDrawBoardTypes from '../../types/RealDrawBoardTypes';
export * as RealRendererTypes from '../../types/RealRendererTypes';
/**
 * Drawing board.
 */
export declare class RealDrawBoard extends RealRenderer<IRealDrawBoardEvents> {
    settings: RealDrawBoardSettings;
    tool: Tool;
    toolSettings: ToolSettings;
    /** key -> identifier, value -> coordinate
     *  For mouse, the key is 'mouse', for touches, stringified identifier -> https://developer.mozilla.org/en-US/docs/Web/API/Touch/identifier
     * @internal
     */
    _lastCoords: Map<string, Coordinate>;
    /** @internal */
    _doPreview: boolean;
    /**
     * The preview for the current stroke
     */
    /** @internal */
    _previewStroke: Map<string, Stroke>;
    /**
     * Map of ongoing strokes to their identifiers
     * key -> identifier, value -> strokeIndex
     * @internal
     */
    _strokeIdentifierMap: Map<string, number>;
    protected _resetBoard: typeof _resetBoard;
    protected _addDOMEvents: typeof _addDOMEvents;
    protected _removeDOMEvents: typeof _removeDOMEvents;
    protected _tools: Tools;
    protected _getMouseCoords: typeof _getMouseCoords;
    protected _getTouchCoords: typeof _getTouchCoords;
    changeToolSetting: typeof changeToolSetting;
    changeTool: typeof changeTool;
    clearPreview: typeof clearPreview;
    clear: typeof clear;
    constructor(options: RealDrawBoardOptions);
    protected _mouseDownEventListener: (e: MouseEvent) => void;
    protected _mouseUpEventListener: (e: MouseEvent) => void;
    protected _mouseLeaveEventListener: (e: MouseEvent) => void;
    protected _mouseMoveEventListener: (e: MouseEvent) => void;
    protected _previewMouseMoveEventListener: (e: MouseEvent) => void;
    protected _wheelEventListener: (e: WheelEvent) => void;
    protected _panEventListener: (e: MouseEvent) => void;
    protected _keyPressEventListener: (e: KeyboardEvent) => void;
    protected _touchStartEventListener: (e: TouchEvent) => void;
    protected _touchEndEventListener: (e: TouchEvent) => void;
    protected _touchMoveEventListener: (e: TouchEvent) => void;
    protected _previewTouchMoveEventListener: (e: TouchEvent) => void;
    startRender: () => any;
    stopRender: () => any;
    reset(): this;
    protected _beforeExport(): void;
}
