"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.Line = exports.LineDefaults = exports.name = void 0;
var circle_1 = require("../../../pathMakers/circle");
var line_1 = require("../../../pathMakers/line");
var getRGBColorString_1 = require("../../../util/getRGBColorString");
var _group_1 = require("../../RealRenderer/strokeNodes/_group");
var _path_1 = require("../../RealRenderer/strokeNodes/_path");
var getRadiusFromThickness_1 = require("./util/getRadiusFromThickness");
var _tool_1 = require("./_tool");
exports.name = 'line';
exports.LineDefaults = {
    lineThickness: 1,
    lineColor: [1, 1, 1]
};
var Line = /** @class */ (function (_super) {
    __extends(Line, _super);
    function Line() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        /** key -> identifier, value -> coordinate
         *  For mouse, the key is 'mouse', for touches, stringified identifier -> https://developer.mozilla.org/en-US/docs/Web/API/Touch/identifier
         */
        _this._startCoords = new Map(); /* key -> identifier, value -> coordinate*/
        return _this;
    }
    Line.prototype._startStroke = function (coords, identifier, target) {
        this.RDB._doPreview = false;
        if (this.RDB._previewStroke.has(identifier)) {
            this.RDB._previewStroke.get(identifier).forEach(function (strokeNode) {
                strokeNode.delete();
            });
        }
        var linePath = new _path_1.Path('', 'strokes');
        linePath.setStroke(getRGBColorString_1.getRGBColorString(this.RDB.toolSettings.lineColor));
        linePath.setStrokeWidth(this.RDB.toolSettings.lineThickness);
        var startCircle = circle_1.getCircleNode(coords, getRadiusFromThickness_1.getRadiusFromThickness(this.RDB.toolSettings.lineThickness), this.RDB.toolSettings.lineColor, 'strokes');
        var endCircle = circle_1.getCircleNode(coords, getRadiusFromThickness_1.getRadiusFromThickness(this.RDB.toolSettings.lineThickness), this.RDB.toolSettings.lineColor, 'strokes');
        this.RDB._addStroke([new _group_1.GroupNode('strokes', [linePath, startCircle, endCircle])]);
        this.RDB._strokeIdentifierMap.set(identifier, this.RDB._strokeIndex);
        this._startCoords.set(identifier, coords);
    };
    Line.prototype._endStroke = function (endCoords, identifier, target) {
        var lineNode = this.RDB.strokes[this.RDB._strokeIdentifierMap.get(identifier)][0];
        lineNode.innerNodes[0].updatePath(line_1.getLinePathCommand(this._startCoords.get(identifier), endCoords));
        lineNode.innerNodes[2].updateCenter(endCoords);
        this._startCoords.delete(identifier);
        this.RDB._doPreview = true;
    };
    Line.prototype._doStroke = function (coords, identifier, target) {
        var lineNode = this.RDB.strokes[this.RDB._strokeIndex][0];
        lineNode.innerNodes[0].updatePath(line_1.getLinePathCommand(this._startCoords.get(identifier), coords));
        lineNode.innerNodes[2].updateCenter(coords);
    };
    Line.prototype._toolPreview = function (coords, identifier, target) {
        if (this.RDB._previewStroke.get(identifier).length == 0) {
            var circleNode = circle_1.getCircleNode(coords, getRadiusFromThickness_1.getRadiusFromThickness(this.RDB.toolSettings.lineThickness), this.RDB.toolSettings.lineColor, 'overlay');
            circleNode.setFill(getRGBColorString_1.getRGBColorString(this.RDB.toolSettings.lineColor));
            circleNode.setStroke(getRGBColorString_1.getRGBColorString(this.RDB.toolSettings.lineColor));
            this.RDB._previewStroke.get(identifier).push(circleNode);
        }
        else {
            var circleNode = this.RDB._previewStroke.get(identifier)[0];
            circleNode.updateCenter(coords);
            circleNode.updateRadius(getRadiusFromThickness_1.getRadiusFromThickness(this.RDB.toolSettings.lineThickness));
            circleNode.setFill(getRGBColorString_1.getRGBColorString(this.RDB.toolSettings.lineColor));
            circleNode.setStroke(getRGBColorString_1.getRGBColorString(this.RDB.toolSettings.lineColor));
        }
    };
    Line.prototype._onScroll = function (scrollDelta, coords, identifier) {
        this.RDB.changeToolSetting('lineThickness', Math.max(1, this.RDB.toolSettings.lineThickness - scrollDelta));
        if (this.RDB._previewStroke.get(identifier) != null && this.RDB._previewStroke.get(identifier).length !== 0) {
            this.RDB._previewStroke.get(identifier)[0].updateRadius(getRadiusFromThickness_1.getRadiusFromThickness(this.RDB.toolSettings.lineThickness));
            this.RDB._display(this.RDB._previewStroke.get(identifier));
        }
    };
    return Line;
}(_tool_1.Tool));
exports.Line = Line;
