"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.TextTool = exports.TextDefaults = exports.name = void 0;
var getRGBColorString_1 = require("../../../../util/getRGBColorString");
var _polygon_1 = require("../../../RealRenderer/strokeNodes/_polygon");
var _text_1 = require("../../../RealRenderer/strokeNodes/_text/_text");
var _tool_1 = require("../_tool");
var _mapKeyToAction_1 = require("./_mapKeyToAction");
exports.name = 'text';
exports.TextDefaults = {
    fontSize: 10,
    fontColor: [1, 1, 1],
    textToolMode: 'new',
    bold: false,
    italic: false,
    underline: false
};
/**
 * Text Tool
 * HOW TO USE:
 * There are two modes: 'new' and 'edit'
 * In 'new' mode, drag on the board to insert new text, this will also change the mode to edit mode.
 * In 'edit' mode, type to enter text. The following keyboard shortcuts/functions are supported:
 * 1) Arrow keys: Move the cursor.
 * 2) Backspace
 * 3) Shift + Enter: Add a new line.
 * 4) Enter: Confirm editing. NOTE: The text will NOT be editable after this step. This will be changed in future version.
 *
 * Cursor Styling:
 * The cursor is currently just a '|' character in an SVG <tspan>.
 * To style the cursor (blinking, color), add CSS to the `.svg-real-db-text-cursor' class.
 * See: /example/index.css in the repo.
 */
var TextTool = /** @class */ (function (_super) {
    __extends(TextTool, _super);
    function TextTool() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        /** key -> identifier, value -> coordinate
         *  For mouse, the key is 'mouse', for touches, stringified identifier -> https://developer.mozilla.org/en-US/docs/Web/API/Touch/identifier
         */
        _this._startCoords = new Map(); /* key -> identifier, value -> coordinate*/
        _this._selectedNode = null;
        return _this;
    }
    TextTool.prototype._onLoad = function () {
        var _this = this;
        this.RDB.on('tool-setting-change', 'text-tool-handler', function (_a) {
            var settingName = _a.settingName, newValue = _a.newValue;
            if (_this.RDB.toolSettings.textToolMode === 'edit' && _this._selectedNode !== null) {
                switch (settingName) {
                    case 'fontColor':
                        _this._selectedNode.setFill(getRGBColorString_1.getRGBColorString(newValue));
                        break;
                    case 'fontSize':
                        _this._selectedNode.setFontSize(newValue);
                        break;
                    case 'bold':
                        _this._selectedNode.setBold(newValue);
                        break;
                    case 'italic':
                        _this._selectedNode.setItalic(newValue);
                        break;
                    case 'underline':
                        _this._selectedNode.setUnderline(newValue);
                        break;
                }
            }
            if (settingName === 'textToolMode' && newValue === 'new') {
                if (_this._selectedNode !== null) {
                    _this._selectedNode.destroyCursor();
                    _this._selectedNode = null;
                }
            }
        });
        this.RDB.on('board-cleared', 'text-tool-handler', function () {
            _this.RDB.changeToolSetting('textToolMode', 'edit');
        });
    };
    TextTool.prototype._onUnload = function () {
        this.RDB.changeToolSetting('textToolMode', 'new');
    };
    TextTool.prototype._startStroke = function (coords, identifier, target) {
        if (this.RDB.toolSettings.textToolMode === 'new') {
            if (this.RDB._previewStroke.has(identifier)) {
                this.RDB._previewStroke.get(identifier).forEach(function (strokeNode) {
                    strokeNode.delete();
                });
            }
            this._startCoords.set(identifier, coords);
            var boundingBox = new _polygon_1.Polygon([coords, coords, coords, coords], 'overlay');
            boundingBox.setDashed(getRGBColorString_1.getRGBColorString(this.RDB.bgColor.map(function (c) { return 1 - c; })));
            boundingBox.setFill('transparent');
            this.RDB._previewStroke.set(identifier, [boundingBox]);
        }
    };
    TextTool.prototype._endStroke = function (endCoords, identifier, target) {
        if (this.RDB.toolSettings.textToolMode === 'new') {
            var _a = this._startCoords.get(identifier), startX = _a[0], startY = _a[1];
            var endX = endCoords[0], endY = endCoords[1];
            var baselineCoords = [
                (startX + endX) / 2,
                (startY + endY) / 2
            ];
            var textPath = new _text_1.Text(baselineCoords, 'Enter Text', 'strokes');
            textPath.setFill(getRGBColorString_1.getRGBColorString(this.RDB.toolSettings.fontColor));
            textPath.setFontSize(this.RDB.toolSettings.fontSize);
            textPath.setBold(this.RDB.toolSettings.bold);
            textPath.setItalic(this.RDB.toolSettings.italic);
            textPath.setUnderline(this.RDB.toolSettings.underline);
            this.RDB._addStroke([textPath]);
            this.RDB._strokeIdentifierMap.set(identifier, this.RDB._strokeIndex);
            if (this._selectedNode !== null)
                this._selectedNode.destroyCursor();
            this._selectedNode = textPath;
            this.RDB.changeToolSetting('textToolMode', 'edit');
            if (this.RDB._previewStroke.has(identifier)) {
                this.RDB._previewStroke.get(identifier).forEach(function (node) { return node.delete(); });
                this.RDB._previewStroke.delete(identifier);
            }
        }
    };
    TextTool.prototype._doStroke = function (coords, identifier, target) {
        if (this.RDB.toolSettings.textToolMode === 'new') {
            var boundingBox = this.RDB._previewStroke.get(identifier)[0];
            var _a = this._startCoords.get(identifier), startX = _a[0], startY = _a[1];
            var endX = coords[0], endY = coords[1];
            boundingBox.updatePoints([
                [startX, startY],
                [endX, startY],
                [endX, endY],
                [startX, endY]
            ]);
        }
    };
    TextTool.prototype._onKey = function (e) {
        if (this.RDB.toolSettings.textToolMode === 'edit' && this._selectedNode !== null) {
            var _a = _mapKeyToAction_1._mapKeyToAction(e, this._selectedNode), _selectedNode = _a[0], doPrevent = _a[1];
            if (doPrevent)
                e.preventDefault();
            this._selectedNode = _selectedNode;
            if (this._selectedNode === null)
                this.RDB.changeToolSetting('textToolMode', 'new');
        }
    };
    TextTool.prototype._onScroll = function (scrollDelta) {
        this.RDB.changeToolSetting('fontSize', Math.max(1, this.RDB.toolSettings.fontSize - scrollDelta / 5));
    };
    return TextTool;
}(_tool_1.Tool));
exports.TextTool = TextTool;
