import { IRealRendererEvents } from '../../constants/events/RealRendererEvents';
import { BGType, Color, GraphDimensions, RealRendererOptions, RealRendererSettings, Stroke, SVGSections } from '../../types/RealRendererTypes';
import { changeBackground, normalizeBG } from './background';
import { draw, _addStroke, _display, _draw, _drawFunc } from './draw/draw';
import { startRender, stopRender, toggleRender, _render } from './draw/render';
import { EventEmitter } from './events/eventEmitter';
import { exportData, getExportVersion, importData } from './import-export';
import { attach, _setSVG, _setViewBox } from './svg/svg-dom';
import { changeOffsets, scale } from './svg/svg-settings';
import { redo, undo } from './undo';
export * from '../../constants/defaults/RealRendererDefaults';
export * as RealRendererTypes from '../../types/RealRendererTypes';
/**
 * General Real Renderer with no specific purpose. Should be extended to use.
 */
export declare abstract class RealRenderer<EventTypes extends IRealRendererEvents = IRealRendererEvents> extends EventEmitter<EventTypes> {
    svg: SVGSVGElement;
    svgSections: SVGSections;
    dimensions: GraphDimensions;
    scaleFactor: number;
    _offsetX: number;
    _offsetY: number;
    originalDimensions: GraphDimensions;
    strokes: Stroke[];
    settings: RealRendererSettings;
    _strokeIndex: number;
    bgColor: Color;
    bgType: BGType;
    drawsPerFrame: number;
    timeStep: number;
    time: number;
    _doRender: boolean;
    undo: typeof undo;
    redo: typeof redo;
    exportData: typeof exportData;
    static getExportVersion: typeof getExportVersion;
    protected _beforeExport(): void;
    importData: typeof importData;
    changeBackground: typeof changeBackground;
    scale: typeof scale;
    changeOffsets: typeof changeOffsets;
    attach: typeof attach;
    draw: typeof draw;
    startRender: typeof startRender;
    stopRender: typeof stopRender;
    toggleRender: typeof toggleRender;
    protected normalizeBG: typeof normalizeBG;
    protected _setSVG: typeof _setSVG;
    protected _setViewBox: typeof _setViewBox;
    protected _drawFunc: typeof _drawFunc;
    protected _draw: typeof _draw;
    /** @internal */
    _display: typeof _display;
    /** @internal */
    _addStroke: typeof _addStroke;
    protected _render: typeof _render;
    /**
     *
     * @param options
     * @param eventList
     */
    constructor(options: RealRendererOptions, eventList?: (keyof EventTypes)[]);
    /**
     * Resets the internal time counter.
     * @returns Self for chaining.
     */
    resetTime(): this;
    /**
     * Resets everything regarding the graph.
     * @returns Self for chaining.
     */
    reset(): this;
}
