"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Node = void 0;
var Node = /** @class */ (function () {
    function Node(section, elementName, strokeNodeType) {
        var path = document.createElementNS('http://www.w3.org/2000/svg', elementName);
        this.strokeNodeType = strokeNodeType;
        this.node = path;
        this.section = section;
    }
    Node.prototype._exportBasicData = function () {
        return {
            stroke: this.node.hasAttribute('stroke') ? this.node.getAttribute('stroke') : null,
            fill: this.node.hasAttribute('fill') ? this.node.getAttribute('fill') : null,
            strokeWidth: this.node.hasAttribute('stroke-width') ? this.node.getAttribute('stroke-width') : null,
            dashed: this.node.hasAttribute('stroke-dasharray') ? { dashColor: this.node.getAttribute('stroke') } : false,
            id: this.node.hasAttribute('id') ? this.node.getAttribute('id') : null,
            class: this.node.hasAttribute('class') ? this.node.getAttribute('class') : null,
            style: this.node.hasAttribute('style') ? this.node.getAttribute('style') : null,
            type: this.strokeNodeType,
            section: this.section
        };
    };
    /** @deprecated */
    Node.prototype.importV1 = function (data) {
        var wrapper = document.createElementNS('http://www.w3.org/2000/svg', 'svg');
        wrapper.innerHTML = data;
        this.node = wrapper.firstChild;
        wrapper.removeChild(this.node);
        wrapper.remove();
    };
    Node.prototype.import = function (data) {
        if (data.stroke !== null)
            this.setStroke(data.stroke);
        if (data.fill !== null)
            this.setFill(data.fill);
        if (data.strokeWidth !== null)
            this.node.setAttribute('stroke-width', data.strokeWidth);
        if (data.dashed !== false)
            this.setDashed(data.dashed.dashColor);
        if (data.id !== null)
            this.setId(data.id);
        if (data.class !== null)
            this.node.setAttribute('class', data.class);
        if (data.style !== null)
            this.node.setAttribute('style', data.style);
    };
    Node.prototype.setStroke = function (stroke) {
        this.node.setAttribute('stroke', stroke);
    };
    Node.prototype.setFill = function (fill) {
        this.node.setAttribute('fill', fill);
    };
    Node.prototype.setStrokeWidth = function (width) {
        this.node.setAttribute('stroke-width', width.toString());
    };
    Node.prototype.setDashed = function (dashColor) {
        this.node.setAttribute('stroke-dasharray', '2,2');
        this.setStroke(dashColor);
    };
    Node.prototype.setId = function (id) {
        this.node.setAttribute('id', id);
    };
    Node.prototype.setClassName = function (className) {
        this.node.classList.add(className);
    };
    Node.prototype.delete = function () {
        this.node.remove();
    };
    return Node;
}());
exports.Node = Node;
