"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Text = void 0;
var _node_1 = require("./_node");
var Text = /** @class */ (function (_super) {
    __extends(Text, _super);
    function Text(position, initialText, section) {
        var _this = _super.call(this, section, 'text', 'text') || this;
        _this.tspans = [];
        /** The text tspan after which the cursor is placed */
        _this.cursorIndex = 0;
        /** span indexes at which a newLine starts */
        _this.lineIndexes = [];
        _this.position = [0, 0];
        var path = document.createElementNS('http://www.w3.org/2000/svg', 'text');
        _this.node = path;
        _this.section = section;
        _this.node.setAttribute('text-anchor', 'middle');
        _this._addTspan(initialText);
        _this.cursorSpan = document.createElementNS('http://www.w3.org/2000/svg', 'tspan');
        _this.node.appendChild(_this.cursorSpan);
        _this.cursorSpan.classList.add('svg-real-db-text-cursor');
        _this.cursorSpan.style.setProperty('position', 'relative');
        _this.cursorSpan.textContent = '|';
        _this.cursorIndex = 0;
        _this._addTspan('', _this.cursorIndex + 1); // span after cursor
        _this.lineIndexes.push(0);
        _this.position = __spreadArrays(position);
        _this.node.setAttribute('x', (_this.position[0]).toString());
        _this.node.setAttribute('y', _this.position[1].toString());
        return _this;
    }
    Text.prototype._addTspan = function (text, index, relativeTo) {
        if (index === void 0) { index = 0; }
        if (relativeTo === void 0) { relativeTo = 'spanAfter'; }
        this.lineIndexes = this.lineIndexes.map(function (i) { return i >= index ? i + 1 : i; });
        this.tspans.splice(index, 0, document.createElementNS('http://www.w3.org/2000/svg', 'tspan'));
        var newTspan = this.tspans[index];
        if (relativeTo === 'spanAfter') {
            if (this.tspans[index + 1])
                this.tspans[index + 1].insertAdjacentElement('beforebegin', newTspan);
            else
                this.node.insertAdjacentElement('beforeend', newTspan);
        }
        else {
            if (this.tspans[index - 1])
                this.tspans[index - 1].insertAdjacentElement('afterend', newTspan);
            else
                this.node.insertAdjacentElement('afterbegin', newTspan);
        }
        newTspan.textContent = text;
        newTspan.style.setProperty('whiteSpace', 'pre');
    };
    Text.prototype._getCurrentSpanText = function () {
        return this.tspans[this.cursorIndex].textContent;
    };
    Text.prototype._updateText = function (newText) {
        this.tspans[this.cursorIndex].textContent = newText;
        if (this.tspans[this.cursorIndex].textContent !== '' && this.cursorIndex !== 0) { // workaround: empty svg tspans act like they do not exist
            this.cursorSpan.setAttribute('dy', "0em");
            this.cursorSpan.removeAttribute('x');
            this.tspans[this.cursorIndex].setAttribute('dy', 1.2 + "em");
            this.tspans[this.cursorIndex].setAttribute('x', this.position[0].toString());
        }
    };
    Text.prototype.getText = function () {
        return this.tspans.map(function (tspan) { return tspan.textContent; }).join('');
    };
    Text.prototype.appendText = function (append) {
        this._updateText(this._getCurrentSpanText() + append);
    };
    Text.prototype.deleteLastCharacter = function () {
        this._updateText(this._getCurrentSpanText().slice(0, -1));
        console.log(this.tspans);
        // end of a newline
        if (this._getCurrentSpanText() === '' && this.cursorIndex > 0) {
            // remove that line
            var removedLine_1 = this.lineIndexes.splice(this.lineIndexes.indexOf(this.cursorIndex), 1)[0];
            this.tspans[removedLine_1].remove();
            this.tspans.splice(removedLine_1, 1);
            this.lineIndexes = this.lineIndexes.map(function (i) { return i >= removedLine_1 ? i - 1 : i; });
            this.cursorIndex = removedLine_1 - 1;
        }
    };
    Text.prototype.updateTextBaseline = function (position) {
        this.node.setAttribute('x', position[0].toString());
        this.node.setAttribute('y', position[1].toString());
        this.tspans.forEach(function (tspan) {
            if (tspan.hasAttribute('x'))
                tspan.setAttribute('x', position[0].toString());
        });
    };
    Text.prototype.setStyle = function (proprty, value) {
        this.node.style[proprty] = value;
    };
    Text.prototype.setFontSize = function (size) {
        this.node.style.fontSize = size + "px";
    };
    Text.prototype.moveCursorLeft = function () {
        if (this.tspans[this.cursorIndex].textContent.length > 0) {
            if (!this.tspans[this.cursorIndex + 1])
                this._addTspan('', this.cursorIndex + 1);
            var beforeCursorText = this.tspans[this.cursorIndex].textContent;
            var afterCursorText = this.tspans[this.cursorIndex + 1].textContent;
            this.tspans[this.cursorIndex + 1].textContent = beforeCursorText[beforeCursorText.length - 1] + afterCursorText; // last character of before text becomes first character of after text
            this.tspans[this.cursorIndex].textContent = beforeCursorText.slice(0, -1); // remove last character
            if (this.tspans[this.cursorIndex].textContent === '' && this.cursorIndex !== 0) { // workaround: empty svg tspans act like they do not exist
                this.cursorSpan.setAttribute('dy', 1.2 + "em");
                this.cursorSpan.setAttribute('x', this.position[0].toString());
            }
        }
        else if (this.cursorIndex > 0) {
            this.tspans[this.cursorIndex + 1].setAttribute('dy', 1.2 + "em");
            this.tspans[this.cursorIndex + 1].setAttribute('x', this.position[0].toString());
            this.cursorIndex = this.lineIndexes[this.lineIndexes.indexOf(this.cursorIndex) - 1];
            this.tspans[this.cursorIndex + 1].setAttribute('dy', '0');
            this.tspans[this.cursorIndex + 1].removeAttribute('x');
            this.cursorSpan.remove();
            this.tspans[this.cursorIndex].insertAdjacentElement('afterend', this.cursorSpan);
            this.cursorSpan.setAttribute('dy', '0');
            this.cursorSpan.removeAttribute('x');
        }
    };
    Text.prototype.moveCursorRight = function () {
        if (this.tspans[this.cursorIndex + 1] && this.tspans[this.cursorIndex + 1].textContent.length > 0) {
            var beforeCursorText = this.tspans[this.cursorIndex].textContent;
            var afterCursorText = this.tspans[this.cursorIndex + 1].textContent;
            this.tspans[this.cursorIndex].textContent = beforeCursorText + afterCursorText[0]; // first character of after text becomes last character of before text
            this.tspans[this.cursorIndex + 1].textContent = afterCursorText.slice(1); // remove first character of after cursorIndextext
            if (this.tspans[this.cursorIndex].textContent !== '' && this.cursorIndex !== 0) { // workaround: empty svg tspans act like they do not exist
                this.cursorSpan.setAttribute('dy', "0em");
                this.cursorSpan.removeAttribute('x');
                this.tspans[this.cursorIndex].setAttribute('dy', 1.2 + "em");
                this.tspans[this.cursorIndex].setAttribute('x', this.position[0].toString());
            }
        }
        else if (this.cursorIndex < this.tspans.length - 2) {
            this.cursorIndex = this.lineIndexes[this.lineIndexes.indexOf(this.cursorIndex) + 1];
            this.cursorSpan.remove();
            this.tspans[this.cursorIndex].insertAdjacentElement('afterend', this.cursorSpan);
            this.cursorSpan.setAttribute('dy', 1.2 + "em");
            this.cursorSpan.setAttribute('x', this.position[0].toString());
            this.tspans[this.cursorIndex + 1].setAttribute('dy', '0');
            this.tspans[this.cursorIndex + 1].removeAttribute('x');
        }
    };
    Text.prototype.newLine = function () {
        if (this.tspans[this.cursorIndex + 1]) {
            // if there is a span after the cursor, linebreak the cursor
            this.cursorSpan.setAttribute('dy', 1.2 + "em");
            this.cursorSpan.setAttribute('x', this.position[0].toString());
            // add a new empty span on that line
            this._addTspan('', this.cursorIndex + 1, 'spanBefore');
            // increment cursorIndex due to new span
            this.cursorIndex++;
            this.lineIndexes.push(this.cursorIndex);
            this.lineIndexes.sort(function (a, b) { return a - b; });
        }
    };
    Text.prototype.destroyCursor = function () {
        this.cursorSpan.remove();
    };
    return Text;
}(_node_1.Node));
exports.Text = Text;
