"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addCursor = exports.destroyCursor = exports.moveCursorUp = exports.moveCursorDown = exports.moveCursorRight = exports.moveCursorLeft = void 0;
var moveOneCharacterLeft = function (_a) {
    var str1 = _a[0], str2 = _a[1];
    return [str1 + str2[0], str2.slice(1)];
};
var moveOneCharacterRight = function (_a) {
    var str1 = _a[0], str2 = _a[1];
    return [str1.slice(0, -1), str1[str1.length - 1] + str2];
};
var editTspanText = function (_a, textChangeFunc) {
    var tspan1 = _a[0], tspan2 = _a[1];
    var _b = textChangeFunc([tspan1.textContent, tspan2.textContent]), newTspan1Text = _b[0], newTspan2Text = _b[1];
    tspan1.textContent = newTspan1Text;
    tspan2.textContent = newTspan2Text;
};
function moveCursorLeft() {
    // if the cursor is in between the line, move the cursor in the text
    if (this.tspans[this.cursorIndex].textContent.length > 0) {
        editTspanText([this.tspans[this.cursorIndex], this.tspans[this.cursorIndex + 1]], moveOneCharacterRight);
        // workaround: empty svg tspans act like they do not exist
        if (this.tspans[this.cursorIndex].textContent === '' && this.cursorIndex !== 0) {
            this._setTspanPositioning(this.cursorSpan, 'lineBreak');
        }
    }
    // if the cursor is at the beginning of the line, move the cursor up
    else if (this.cursorIndex > 0) {
        this._setTspanPositioning(this.tspans[this.cursorIndex + 1], 'lineBreak');
        this.cursorIndex = this.lineIndexes[this.lineIndexes.indexOf(this.cursorIndex) - 1];
        this._setTspanPositioning(this.tspans[this.cursorIndex + 1], 'inline');
        this.cursorSpan.remove();
        this.tspans[this.cursorIndex].insertAdjacentElement('afterend', this.cursorSpan);
        this._setTspanPositioning(this.cursorSpan, 'inline');
        this.lineIndexes[this.lineIndexes.indexOf(this.cursorIndex + 1)] = this.cursorIndex + 2;
    }
}
exports.moveCursorLeft = moveCursorLeft;
function moveCursorRight() {
    if (this.tspans[this.cursorIndex + 1] !== undefined && this.tspans[this.cursorIndex + 1].textContent.length > 0) {
        editTspanText([this.tspans[this.cursorIndex], this.tspans[this.cursorIndex + 1]], moveOneCharacterLeft);
        // workaround: empty svg tspans act like they do not exist
        if (this.tspans[this.cursorIndex].textContent !== '' && this.cursorIndex !== 0) {
            this._setTspanPositioning(this.cursorSpan, 'inline');
            this._setTspanPositioning(this.tspans[this.cursorIndex], 'lineBreak');
        }
    }
    else if (this.cursorIndex < this.tspans.length - 2) {
        this.cursorIndex = this.lineIndexes[this.lineIndexes.indexOf(this.cursorIndex) + 1] - 1;
        this.cursorSpan.remove();
        this.tspans[this.cursorIndex].insertAdjacentElement('afterend', this.cursorSpan);
        this._setTspanPositioning(this.cursorSpan, 'lineBreak');
        this._setTspanPositioning(this.tspans[this.cursorIndex + 1], 'inline');
        this.lineIndexes[this.lineIndexes.indexOf(this.cursorIndex + 1)] = this.cursorIndex;
    }
}
exports.moveCursorRight = moveCursorRight;
function moveCursorDown() {
    // check if another line exists
    var newLineIndex = this.lineIndexes[this.lineIndexes.indexOf(this.cursorIndex) + 1];
    if (newLineIndex !== undefined) {
        // merge the current line into a single tspan
        var beforeCursorText = this.tspans[this.cursorIndex].textContent;
        var afterCursorText = this.tspans[this.cursorIndex + 1].textContent;
        // put all the text in one tspan
        this.tspans[this.cursorIndex].textContent = beforeCursorText + afterCursorText;
        // find out how the next line should be split
        var newLineText = this.tspans[newLineIndex].textContent;
        var newLineLength = newLineText.length;
        var splitIndex = Math.min(beforeCursorText.length - 1, newLineLength - 1);
        // find the split text
        var newSplitText = [
            newLineText.slice(0, splitIndex + 1),
            newLineText.slice(splitIndex + 1)
        ];
        // put the text in the right place
        this.tspans[newLineIndex - 1].textContent = newSplitText[0];
        this.tspans[newLineIndex].textContent = newSplitText[1];
        // move the cursor
        this.cursorSpan.remove();
        this.tspans[newLineIndex - 1].insertAdjacentElement('afterend', this.cursorSpan);
        // fix the dy
        this._setTspanPositioning(this.tspans[newLineIndex], 'inline');
        // workaround: empty svg tspans act like they do not exist
        if (newSplitText[0] === '')
            this._setTspanPositioning(this.cursorSpan, 'lineBreak');
        else
            this._setTspanPositioning(this.tspans[newLineIndex - 1], 'lineBreak');
        // fix the lineIndex
        this.lineIndexes[this.lineIndexes.indexOf(newLineIndex)] = newLineIndex - 1;
        this.lineIndexes.sort(function (a, b) { return a - b; });
        this.cursorIndex = newLineIndex - 1;
    }
}
exports.moveCursorDown = moveCursorDown;
function moveCursorUp() {
    // check if another line exists
    var newLineIndex = this.lineIndexes[this.lineIndexes.indexOf(this.cursorIndex) - 1];
    if (this.lineIndexes.includes(newLineIndex)) {
        // merge the current line into a single tspan
        var beforeCursorText = this.tspans[this.cursorIndex].textContent;
        var afterCursorText = this.tspans[this.cursorIndex + 1].textContent;
        // put all the text in one tspan
        this.tspans[this.cursorIndex + 1].textContent = beforeCursorText + afterCursorText;
        // find out how the next line should be split
        var newLineText = this.tspans[newLineIndex].textContent;
        var newLineLength = newLineText.length;
        var splitIndex = Math.min(beforeCursorText.length - 1, newLineLength - 1);
        // find the split text
        var newSplitText = [
            newLineText.slice(0, splitIndex + 1),
            newLineText.slice(splitIndex + 1)
        ];
        // put the text in the right place
        this.tspans[newLineIndex].textContent = newSplitText[0];
        this.tspans[newLineIndex + 1].textContent = newSplitText[1];
        // move the cursor
        this.cursorSpan.remove();
        this.tspans[newLineIndex].insertAdjacentElement('afterend', this.cursorSpan);
        // fix the dy
        this._setTspanPositioning(this.tspans[newLineIndex + 1], 'inline');
        this._setTspanPositioning(this.tspans[this.cursorIndex + 1], 'lineBreak');
        // fix the lineIndex
        this.lineIndexes[this.lineIndexes.indexOf(this.cursorIndex)] = this.cursorIndex + 1;
        this.lineIndexes.sort(function (a, b) { return a - b; });
        this.cursorIndex = newLineIndex;
    }
}
exports.moveCursorUp = moveCursorUp;
function destroyCursor() {
    // if the cursor was acting as the newline, transfer newline to next tspan
    if (this.cursorSpan.hasAttribute('dy'))
        this._setTspanPositioning(this.tspans[this.cursorIndex + 1], 'lineBreak');
    this.cursorSpan.remove();
}
exports.destroyCursor = destroyCursor;
function addCursor() {
    this.tspans[this.cursorIndex].insertAdjacentElement('afterend', this.cursorSpan);
}
exports.addCursor = addCursor;
